package com.fs.course.mapper;

import java.util.List;

import com.fs.course.domain.FsUserCoursePeriod;
import com.fs.course.param.CompanyRedPacket;
import com.fs.course.vo.FsUserCoursePeriodVO;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

/**
 * 会员营期Mapper接口
 *
 * @author fs
 * @date 2025-04-15
 */
public interface FsUserCoursePeriodMapper
{
    /**
     * 查询会员营期
     *
     * @param periodId 会员营期ID
     * @return 会员营期
     */
    public FsUserCoursePeriod selectFsUserCoursePeriodById(Long periodId);

    /**
     * 查询会员营期列表
     *
     * @param fsUserCoursePeriod 会员营期
     * @return 会员营期集合
     */
    public List<FsUserCoursePeriod> selectFsUserCoursePeriodList(FsUserCoursePeriod fsUserCoursePeriod);

    /**
     * 新增会员营期
     *
     * @param fsUserCoursePeriod 会员营期
     * @return 结果
     */
    public int insertFsUserCoursePeriod(FsUserCoursePeriod fsUserCoursePeriod);

    /**
     * 修改会员营期
     *
     * @param fsUserCoursePeriod 会员营期
     * @return 结果
     */
    public int updateFsUserCoursePeriod(FsUserCoursePeriod fsUserCoursePeriod);

    /**
     * 删除会员营期
     *
     * @param periodId 会员营期ID
     * @return 结果
     */
    public int deleteFsUserCoursePeriodById(Long periodId);

    /**
     * 批量删除会员营期
     *
     * @param periodIds 需要删除的数据ID
     * @return 结果
     */
    public int deleteFsUserCoursePeriodByIds(Long[] periodIds);

    /**
     * 查询主页分页列表
     * @return
     */
    List<FsUserCoursePeriodVO> selectFsUserCoursePeriodPage(FsUserCoursePeriod fsUserCoursePeriod);

    /**
     * 查询训练营营期
     * @param id    训练营ID
     * @return  list
     */
    @Select("select * from fs_user_course_period where training_camp_id = #{id}")
    List<FsUserCoursePeriod> selectFsUserCoursePeriodListByTrainingCampId(@Param("id") Long id);

    List<CompanyRedPacket> selectCompanyByPeriod(@Param("periodId")Long periodId, @Param("videoId") Long videoId);
}
